<?php
/*
 * JCK - Joomla 1.5
 *
 * Copyright (C) 2008 Nick Miles - Database Developments Ltd
 *
 * == BEGIN LICENSE ==
 *
 * Licensed under the terms of any of the following licenses:
 *
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 *
 * == END LICENSE ==
*/

// Do not allow direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.event.plugin');

class plgEditorJCK extends JPlugin
{
	/**
	 * Constructor
	 *
	 * For php4 compatability we must not use the __constructor as a constructor for plugins
	 * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	 * This causes problems with cross-referencing necessary for the observer design pattern.
	 *
	 * @param 	object $subject The object to observe
	 * @param 	array  $config  An array that holds the plugin configuration
	 * @since 1.5
	 */
	function plgEditorJCK(& $subject, $config) {
		parent::__construct($subject, $config);
	}

	function onInit()
	{

		global $mainframe;
		
		$url = JURI::root();

		$return = "<script type=\"text/javascript\" src=\"" . $url . "plugins/editors/jck/fckeditor.js\"></script>";
		
		return $return;

	}

	/**
	 * JCK Editor - get the editor content
	 *
	 * @param string 	The name of the editor
	 */
	function onGetContent( $editor ) {
		return "GetEditorHTML();";
	}

	/**
	 * JCK Editor - set the editor content
	 *
	 * @param string 	The name of the editor
	 */
	function onSetContent( $editor, $html ) {
		return "GetEditorHTML($html);";
	}

	/**
	 * JCK Editor - copy editor content to form field
	 *
	 * @param string 	The name of the editor
	 */
	function onSave( $editor ) {
		return "SaveEditorHTML();";
	}

	/**
	 * JCK Editor - display the editor
	 *
	 * @param string The name of the editor area
	 * @param string The content of the field
	 * @param string The width of the editor area
	 * @param string The height of the editor area
	 * @param int The number of columns for the editor area
	 * @param int The number of rows for the editor area
	 * @param mixed Can be boolean or array.
	 */
	function onDisplay( $name, $content, $width, $height, $col, $row, $buttons = true)
	{
		global $mainframe;

		$db			=& JFactory::getDBO();
		$language	=& JFactory::getLanguage();
		$url 		= JURI::root();

		$toolbar = $this->params->get( 'toolbar', 'Default' );
		$content_css = $this->params->get( 'content_css', 0 );
		$content_css_custom = $this->params->get( 'content_css_custom', '' );
		
		if ( $content_css ) {

			$query = 'SELECT template'
			. ' FROM #__templates_menu'
			. ' WHERE client_id = 0'
			. ' AND menuid = 0'
			;
			$db->setQuery( $query );
			$template = $db->loadResult();		
			
			$file = JPATH_SITE . "/templates/" . $template . "/css/template.css";
			if ( file_exists( $file ) ) {
				$content_css = 'templates/'.$template.'/css/template.css';
			} else {
				$content_css = 'plugins/editors/jck/editor/css/fck_editorarea.css';
			}
		} else {
			if ( $content_css_custom ) {
				$content_css = $content_css_custom;
			} else {
				$content_css = 'plugins/editors/jck/editor/css/fck_editorarea.css';
			}
		}	

		// Write extra session record to enable security if required
		$ip = md5($_SERVER['REMOTE_ADDR']);
		$query ='select session_id from #__session where session_id =\'' .$ip .'\''; 
		$db->setQuery( $query);
		$ip_recorded = $db->loadResult();

		if(!isset($ip_recorded)) // If not recorded, record it
		{
		   	$query = 'insert into #__session(time,session_id,gid,client_id,data) values(\'' . (time() + 7200) . '\',\'' . $ip  . '\',0,0,\''
			. session_id() . '\')';
			$db->Execute( $query );
		} else { // update time and  user session_id 
		   	$query = 'update #__session set time = \'' . (time() + 7200) . '\',data = \'' . session_id() . '\' ' 
			.'where session_id =\'' .$ip .'\''; 
			$db->Execute( $query );  
	                     
		}		
	
		// Only add "px" to width and height if they are not given as a percentage
		if (is_numeric( $width )) {
			$width .= 'px';
		}
		if (is_numeric( $height )) {
			$height .= 'px';
		}
		
		$path = JPATH_SITE;

		$buttons = $this->_displayButtons($name, $buttons);

		$return = "
<textarea id=\"$name\" name=\"$name\" cols=\"$col\" rows=\"$row\" style=\"width:{$width}; height:{$height};\" class=\"mce_editable\">$content</textarea>\n" . $buttons . "
		
<script type=\"text/javascript\">
	
	var oFCKeditor = new FCKeditor('" . $name . "');
	oFCKeditor.BasePath = '" . $url . "plugins/editors/jck/' ;
	oFCKeditor.Width = '" . $width . "' ;
	oFCKeditor.Height = '" . $height . "' ;
	oFCKeditor.ToolbarSet = '" . $toolbar . "' ;
	oFCKeditor.Config['EditorAreaCSS'] = '" . $url . $content_css . "' ;
	oFCKeditor.Config['StylesXmlPath'] = '" . $url . "plugins/editors/jckstyles.xml.php?css_filename=$content_css' ;
	oFCKeditor.Config['CustomConfigurationsPath'] = '" . $url . "plugins/editors/jck.js.php' ;

	function DisplayEditor() {
		oFCKeditor.ReplaceTextarea();
	}
	
	function GetEditorHTML() {
		var oEditor = FCKeditorAPI.GetInstance('" . $name . "');
		return oEditor.GetData();
	}
	
	function SetEditorHTML(html) {
		var oEditor = FCKeditorAPI.GetInstance('" . $name . "');
		oEditor.SetData(html);
		return true;
	}
	
	function InsertEditorHTML(html) {
		var oEditor = FCKeditorAPI.GetInstance('" . $name . "');
		oEditor.InsertHtml(html);
	}
	
	function SaveEditorHTML() {
		var oEditor = FCKeditorAPI.GetInstance('" . $name . "');
		oEditor.UpdateLinkedField();
	}
	
	window.addEvent('domready', DisplayEditor);
	
</script>
";
		
		return $return;
	}
	
	function onGetInsertMethod($name)
	{
		$doc = & JFactory::getDocument();

		$js= "		
	function jInsertEditorText(html) {
		InsertEditorHTML(html);
	}
		";
		
		$doc->addScriptDeclaration($js);
		
		return true;
	}
	
	function _displayButtons($name, $buttons)
	{
		// Load modal popup behavior
		JHTML::_('behavior.modal', 'a.modal-button');

		$args['name'] = $name;
		$args['event'] = 'onGetInsertMethod';

		$return = '';
		$results[] = $this->update($args);
		foreach ($results as $result) {
			if (is_string($result) && trim($result)) {
				$return .= $result;
			}
		}

		if(!empty($buttons))
		{
			$results = $this->_subject->getButtons($name, $buttons);

			/*
			 * This will allow plugins to attach buttons or change the behavior on the fly using AJAX
			 */
			$return .= "\n<div id=\"editor-xtd-buttons\">\n";
			foreach ($results as $button)
			{
				/*
				 * Results should be an object
				 */
				if ( $button->get('name') ) 
				{
					$modal		= ($button->get('modal')) ? 'class="modal-button"' : null;
					$href		= ($button->get('link')) ? 'href="'.$button->get('link').'"' : null;
					$onclick	= ($button->get('onclick')) ? 'onclick="'.$button->get('onclick').'"' : null;
					$return .= "<div class=\"button2-left\"><div class=\"".$button->get('name')."\"><a ".$modal." title=\"".$button->get('text')."\" ".$href." ".$onclick." rel=\"".$button->get('options')."\">".$button->get('text')."</a></div></div>\n";
				}
			}
			$return .= "</div>\n";
		}
		
		return $return;
	}
}
?>